﻿using Microsoft.Extensions.Logging;
using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.BusinessLogicsContracts;
using RenovationWorkContracts.SearchModels;
using RenovationWorkContracts.StoragesContracts;
using RenovationWorkContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RenovationWorkBusinessLogic.BusinessLogics
{
    public class RepairLogic : IRepairLogic
    {
        private readonly ILogger _logger;
        private readonly IRepairStorage _RepairStorage;
        public RepairLogic(ILogger<RepairLogic> logger, IRepairStorage RepairStorage)
        {
            _logger = logger;
            _RepairStorage = RepairStorage;
        }
        public List<RepairViewModel>? ReadList(RepairSearchModel? model)
        {
            _logger.LogInformation("ReadList. RepairName:{RepairName}.Id:{ Id}", model?.RepairName, model?.Id);
            var list = model == null ? _RepairStorage.GetFullList() : _RepairStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public RepairViewModel? ReadElement(RepairSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. RepairName:{RepairName}.Id:{ Id}", model.RepairName, model.Id);
            var element = _RepairStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(RepairBindingModel model)
        {
            CheckModel(model);
            if (_RepairStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(RepairBindingModel model)
        {
            CheckModel(model);
            if (_RepairStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(RepairBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_RepairStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(RepairBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.RepairName))
            {
                throw new ArgumentNullException("Нет названия ремотной работы", nameof(model.RepairName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена ремонтной работы должна быть больше 0", nameof(model.Price));
            }
            if (model.RepairComponents == null || model.RepairComponents.Count == 0)
            {
                throw new ArgumentNullException("Перечень расходых материалов не может быть пустым", nameof(model.RepairComponents));
            }
            _logger.LogInformation("Repair. RepairName:{RepairName}.Price:{Price}.Id: { Id}", model.RepairName, model.Price, model.Id);
            var element = _RepairStorage.GetElement(new RepairSearchModel
            {
                RepairName = model.RepairName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Ремонтная работа с таким названием уже есть");
            }
        }
    }
}
